require 'spec_helper'

describe BaseResource do

  context 'Private Methods' do

    before(:each) do
      @cut = BaseResource.new({ use_ssl: false, server: 'localhost:3000', api_token: 'sltc_api_token'})
    end

    context '#invoke_rest' do

      it "should return a raw string if a non-JSON response is made from the block" do
        result = @cut.send(:invoke_rest, false) do
          "abc123"
        end
        expect(result).to eq("abc123")
      end

      it "should parse a response as JSON by default" do
        result = @cut.send(:invoke_rest) do
          '{ "message": "from the other side" }'
        end
        expect(result).to eq({ 'message' => 'from the other side' })
      end

      it "should output an error message and exit if the JSON payload is invalid" do
        expect {
          @cut.send(:invoke_rest) do
            '{ message => 3'
          end
        }.to raise_error(SystemExit)
      end

      it "should output an error message and exit if there is a non-403,404 exception" do
        mock_response = double("Response")
        expect(mock_response).to receive(:code).at_least(1).and_return(500)
        expect(mock_response).to receive(:description).and_return("System error")

        expect {
          @cut.send(:invoke_rest) do
            raise RestClient::Exception.new(mock_response, 500)
          end
        }.to raise_error(SystemExit)
      end

      it "should output an error message and exit if there is a 403 exception with no json" do
        mock_response = double("Response")
        expect(mock_response).to receive(:code).at_least(1).and_return(403)
        expect(mock_response).to receive(:description).and_return("Some other error")

        expect {
          @cut.send(:invoke_rest, false) do
            raise RestClient::Exception.new(mock_response, 403)
          end
        }.to raise_error(SystemExit)
      end

      it "should output an error message and exit if there is a 404 exception with error json" do
        mock_response = double("Response")
        expect(mock_response).to receive(:code).at_least(1).and_return(404)
        expect(mock_response).to receive(:description).and_return("Some other error")
        expect(mock_response).to receive(:http_body).at_least(:once).and_return('{ "error" : "Error message" }')

        expect {
          @cut.send(:invoke_rest) do
            raise RestClient::Exception.new(mock_response, 404)
          end
        }.to raise_error(SystemExit)
      end

      it "should output an error message and exit if there is a 403 exception with validation error json" do
        mock_response = double("Response")
        expect(mock_response).to receive(:code).at_least(1).times.and_return(403)
        expect(mock_response).to receive(:description).and_return("Some other error")
        expect(mock_response).to receive(:http_body).at_least(:once).and_return('{ "validation_error" : { "name" : [ "Error 1", "Error 2" ] } }')

        expect {
          @cut.send(:invoke_rest) do
            raise RestClient::Exception.new(mock_response, 403)
          end
        }.to raise_error(SystemExit)
      end

      it "should output an error message and exit if there is a 403 exception with some other error json" do
        mock_response = double("Response")
        expect(mock_response).to receive(:code).at_least(1).times.and_return(403)
        expect(mock_response).to receive(:description).and_return("Some other error")
        expect(mock_response).to receive(:http_body).at_least(:once).and_return('{ "other_error" : { "name" : [ "Error 1", "Error 2" ] } }')

        expect {
          @cut.send(:invoke_rest) do
            raise RestClient::Exception.new(mock_response, 403)
          end
        }.to raise_error(SystemExit)
      end

      it "should output an error message and exit if there is a 403 exception with invalid validation error json" do
        mock_response = double("Response")
        expect(mock_response).to receive(:code).at_least(1).times.and_return(403)
        expect(mock_response).to receive(:description).and_return("Some other error")
        expect(mock_response).to receive(:http_body).at_least(:once).and_return('{ "validation_error" : { "name" :  "Error 1", "Error 2" ] } }')

        expect {
          @cut.send(:invoke_rest) do
            raise RestClient::Exception.new(mock_response, 403)
          end
        }.to raise_error(SystemExit)
      end

      it "should output an error message and exit if there is an unknown exception raised in the block" do
        expect {
          @cut.send(:invoke_rest) do
            raise "Some unknown exception"
          end
        }.to raise_error(SystemExit)
      end

    end

    context '#get' do

      it "should return the response from the GET call" do
        expect(@cut).to receive(:invoke_rest).with(true).and_yield

        expect(RestClient).to receive(:get).
          with('http://localhost:3000/tubes', {"Authorization"=>"Token token=\"sltc_api_token\"" }).
          and_return({ hey: 'guy' })

        expect(@cut.send(:get, 'tubes')).to eq({ hey: 'guy' })
      end

    end

    context '#delete' do

      it "should return the response from the DELETE call" do
        expect(@cut).to receive(:invoke_rest).with(true).and_yield

        expect(RestClient).to receive(:delete).
          with('http://localhost:3000/tubes', {"Authorization"=>"Token token=\"sltc_api_token\"" }).
          and_return({ hey: 'guy' })

        expect(@cut.send(:delete, 'tubes')).to eq({ hey: 'guy' })
      end

    end

    context '#post' do

      it "should return the response from the POST call" do
        expect(@cut).to receive(:invoke_rest).with(true).and_yield

        expect(RestClient).to receive(:post).
          with('http://localhost:3000/tubes',
               {"param1"=>"pvalue"},
               {"Authorization"=>"Token token=\"sltc_api_token\"" }).
          and_return({ hey: 'guy' })

        expect(@cut.send(:post, 'tubes', { 'param1' => 'pvalue' })).to eq({ hey: 'guy' })
      end

    end

    context '#put' do

      it "should return the response from the PUT call" do
        expect(@cut).to receive(:invoke_rest).with(true).and_yield

        expect(RestClient).to receive(:put).
          with('http://localhost:3000/tubes',
               {"param1"=>"pvalue"},
               {"Authorization"=>"Token token=\"sltc_api_token\"" }).
          and_return({ hey: 'guy' })

        expect(@cut.send(:put, 'tubes', { 'param1' => 'pvalue' })).to eq({ hey: 'guy' })
      end

    end

    context '#build_url' do

      it "should return the properly formatted url" do
        @cut.options[:use_ssl] = false
        @cut.options[:server] = 'www.bob.com'

        expect(@cut.send(:build_url, 'the_uri')).to eq('http://www.bob.com/the_uri')

        @cut.options[:use_ssl] = true

        expect(@cut.send(:build_url, 'the_uri')).to eq('https://www.bob.com/the_uri')
      end

    end

    context '#authentication_headers' do

      it "should return a hash containing the authentication headers" do
        @cut.options[:api_token] = 'this is the token string'
        expect(@cut.send(:authentication_headers)).
          to eq( { 'Authorization' => "Token token=\"this is the token string\"" } )
      end

    end

  end

end
